;**************************************************************************
;* FileName	: BASE.INC
;* Date 	: 14.01.2008
;* File version : v1.17
								nolist
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.atw.hu
;* E-mail	: majki.mester@gmail.com
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Base
;*		  ==================================
;*		  MULTIPAGE
;*		  EXPAND_MACROS
;*		  XCHG
;*		  XCHGW
;*		  MEM_CLEAR
;*		  GOTO_IF_0
;*		  GOTO_IF_1
;*		  RETURN_IF_0
;*		  RETURN_IF_1
;*		  IT_ENABLE
;*		  IT_DISABLE
;*		  LOAD_ADDRESS
;*		  BANK0, BANK1, BANK2, BANK3
;*		  BIT_SET,    BIT_CLEAR
;*		  BIT_TOGGLE, BIT_TOGGLEW
;*		  BIT_COPY,   BIT_INVCOPY
;*		  IT_PORTB_DISABLE, IT_PORTB_ENABLE, IT_PORTB_FLAG
;*		  IT_TMR0_DISABLE,  IT_TMR0_ENABLE,  IT_TMR0_FLAG
;*		  IT_EXT_DISABLE,   IT_EXT_ENABLE,   IT_EXT_FLAG
;*		  IT_PIE_DISABLE,   IT_PIE_ENABLE
;*		  TABLE_READ
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;*
;*  <none>
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;*  <none>
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  10/02/2001 	Initial Release
;* 1.01	  23/08/2001 	Minor changes (labels)
;* 1.02	  03/11/2001	Add: INCLUDEPATH
;* 1.03   04/04/2002	New Macros: clrw, XCHGW, XCHG
;* 1.04   18/06/2002	Support for the multipage program memory
;* 1.05   10/02/2003    Minor changes
;* 1.06   27/05/2003    New Macros: GOTO_IF_x, RETURN_IF_x
;* 1.07   24/07/2003    New Macros: IT_ENABLE, IT_DISABLE
;* 1.08   08/08/2003    New Macros: LOAD_ADDRESS, BANKx, LL
;* 1.09   14/12/2003    Minor changes
;* 1.10   29/03/2004	Fix MULTIPAGE macro
;* 1.11   30/03/2004	Fix MULTIPAGE macro
;* 1.12   04/01/2005    Fix nop2 for PIC18xxx
;* 1.13	  21/08/2006	New Macros: BIT_SET, BIT_CLEAR, 
;*			            BIT_TOGGLE, BIT_TOGGLEW
;* 1.14   23/08/2006	New Macros: BIT_COPY, BIT_INVCOPY
;* 1.15	  23/11/2006	New Macros: IT_*_ENABLE, IT_*_DISABLE, TI_*_FLAG
;* 1.16   23/06/2007    New Macro: TABLE_READ
;* 1.17   14/01/2008    Support for 12 bit devices
;*
;**************************************************************************


;***** DEFINES ************************************************************

#define		MHz		* 1000000
#define		kHz		* 1000

#define		LL		7Fh &

#define		INCLUDEPATH	d:\work\pic\modules

; ----- basic flags -------------------------------------------------------
#define		FLAG_C		STATUS, C
#define		FLAG_Z		STATUS, Z
#define		FLAG_DC		STATUS, DC

; ----- macro expansion modes ---------------------------------------------

#define		EXP_NONE	0
#define		EXP_COND	1
#define		EXP_ALL		2

#define		EXP		expand
#define		NOEXP		noexpand

; ----- instruction redefines ---------------------------------------------
#define		bank0		bcf	STATUS, RP0
#define		bank1		bsf	STATUS, RP0
#define		bankLO		bcf	STATUS, RP1
#define		bankHI		bsf	STATUS, RP1
#define		ibankLO		bcf	STATUS, IRP
#define		ibankHI		bsf	STATUS, IRP

#define         skipz		btfss	STATUS, Z
#define         skipnz		btfsc	STATUS, Z
#define         skipc		btfss	STATUS, C
#define         skipnc		btfsc	STATUS, C
#define         skipdc		btfss	STATUS, DC
#define         skipndc		btfsc	STATUS, DC

#define		jz		bz
#define		jnz		bnz
#define		jc		bc
#define		jnc		bnc
#define		jdc		bdc
#define		jndc		bndc

#define		testf		tstf
#define		testw		xorlw	0

#define		decw		addlw	-1
#define		incw		addlw	1
#define		clrw		movlw	0

#define		clrz		bcf	STATUS, Z
#define		setz		bsf	STATUS, Z
#define		clrc		bcf	STATUS, C
#define		setc		bsf	STATUS, C
#define		clrdc		bcf	STATUS, DC
#define		setdc		bsf	STATUS, DC

; ----- interrupt support -------------------------------------------------
#define		IT_PORTB_ENABLE	bsf	INTCON, RBIE
#define		IT_PORTB_DISABLE bcf	INTCON, RBIE
#define		IT_PORTB_FLAG	INTCON, RBIF

#define		IT_EXT_ENABLE	bsf	INTCON, INTE
#define		IT_EXT_DISABLE	bcf	INTCON, INTE
#define		IT_EXT_FLAG	INTCON, INTF

#define		IT_TMR0_ENABLE	bsf	INTCON, T0IE
#define		IT_TMR0_DISABLE	bcf	INTCON, T0IE
#define		IT_TMR0_FLAG	INTCON, T0IF

#define		IT_PIE_ENABLE	bsf	INTCON, PEIE
#define		IT_PIE_DISABLE	bcf	INTCON, PEIE


; ----- multipage support -------------------------------------------------
#define		fcall		call	; far call
#define		fgoto		goto	; far goto
#define		NUMBER_OF_PAGE	1
#define		NUMBER_OF_BANK  2
		variable PAGESIZE_BITS = 11	; default for 14 bit devices



;***** nop2 ***************************************************************
;* 2 clock length no operation
;* Param : -            			;
;* Mem   : 1
;* Clock : 2
;**************************************************************************
nop2		macro
		local	nop_label
		goto	nop_label
nop_label
		endm



;***** MULTIPAGE **********************************************************
;* support multipage program memory   
;* Param : numofpage            		; number of the pages (1,2,4)
;* 	   pagesize				; size of the pages (512, 2048)
;*	   numofbank				; amount number of banks (2,4)
;* Mem   : -
;* Clock : -
;**************************************************************************
MULTIPAGE	macro numofpage, pagesize, numofbank
									NOEXP
		local size

	if (numofpage!=1)&&(numofpage!=2)&&(numofpage!=4)
	  error Value of variable "numofpage" is false!
	endif

	if (pagesize!=512)&&(pagesize!=2048)
	  error Value of variable "pagesize" is false!
	endif

	if (numofbank!=1)&&(numofbank!=2)&&(numofbank!=4)
	  error Value of variable "numofbank" is false!
	endif


#undefine	NUMBER_OF_PAGE
#define		NUMBER_OF_PAGE	numofpage
#undefine	NUMBER_OF_BANK
#define		NUMBER_OF_BANK	numofbank

PAGESIZE_BITS=0
size=pagesize>>1
	while ((size)!=0)
size>>=1
PAGESIZE_BITS++
	endw

	if (NUMBER_OF_PAGE!=1)
#undefine	fcall
#define		fcall		lcall	; far call 
#undefine	fgoto
#define		fgoto		lgoto	; far goto
	endif


	if (NUMBER_OF_BANK==1)
#define	BANK0		
#define	BANK1		
#define	BANK2	
#define	BANK3
	endif

	if (NUMBER_OF_BANK==2)
#define	BANK0		bank0
#define	BANK1		bank1
#define	BANK2		bank0
#define	BANK3		bank1
	endif

	if (NUMBER_OF_BANK==4)
#define	BANK0		_BANK0
#define	BANK1		_BANK1
#define	BANK2		_BANK2
#define	BANK3		_BANK3
	endif
									EXP


		endm



;***** EXPAND_MACROS ******************************************************
;* set macro expansion mode   
;* Param : mode            		; EXP_NONE, EXP_COND, EXP_ALL
;* Mem   : -
;* Clock : -
;**************************************************************************
EXPAND_MACROS	macro	mode
									NOEXP
#undefine	EXP
#undefine	NOEXP	

	if (mode==EXP_NONE)
#define		EXP		noexpand
#define		NOEXP		noexpand
	endif
								
	if (mode==EXP_COND)
#define		EXP		expand
#define		NOEXP		noexpand
	endif

	if (mode==EXP_ALL)
#define		EXP		expand
#define		NOEXP		expand
	endif
									EXP
		endm



;***** XCHGW **************************************************************
;* eXchange file registers (W <-> file1)
;* Param : file1            		; 
;* Mem   : 3
;* Clock : 3
;**************************************************************************
XCHGW		macro file1
		;==========================================================
			xorwf	file1,f
			xorwf	file1,w
			xorwf	file1,f


		endm


;***** XCHG ***************************************************************
;* eXchange file registers (file1 <-> file2)
;* Param : file1            		; 
;*         file2			; 
;* Mem   : 4
;* Clock : 4
;**************************************************************************
XCHG		macro file1, file2
		;==========================================================
			movf	file1,w                 
			subwf	file2,w
			addwf	file1,f
			subwf	file2,f


		endm



;***** MEM_CLEAR macro ****************************************************
;* Clear file-memory array 
;* Param : addr             		; start address of mem.
;*         count			; number of bytes must be clear
;* Mem   : 8-9
;* Clock : 6*count + 4   
;**************************************************************************
MEM_CLEAR	macro	addr, count
		;==========================================================
									NOEXP
			local MEM_l1
			if ((NUMBER_OF_BANK>2)&&(addr>0FFh))
									EXP								   EXP
			  ibankHI
									NOEXP
			endif
									EXP
			movlw	addr & 0FFh	; clear header
			movwf	FSR
			movlw	count
MEM_l1
			clrf	INDF
			incf	FSR,f
			decw
			jnz	MEM_l1
									NOEXP
			if ((NUMBER_OF_BANK>2)&&(addr>0FFh))
									EXP
			  ibankLO
									NOEXP
		  	endif
									EXP


		endm




;***** GOTO_IF_0 **********************************************************
;* Jump to <label> if the <bit> of the <filereg> is 0.
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;*         label			; destination label
;* Mem   : 2
;* Clock : 2
;**************************************************************************
GOTO_IF_0	macro filereg, bit, label
		;==========================================================
			btfss   filereg, bit         
			goto	label


		endm




;***** GOTO_IF_1 **********************************************************
;* Jump to <label> if the <bit> of the <filereg> is 1.
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;*         label			; destination label
;* Mem   : 2
;* Clock : 2
;**************************************************************************
GOTO_IF_1	macro filereg, bit, label
		;==========================================================
			btfsc   filereg, bit         
			goto	label


		endm



;***** RETURN_IF_0 ********************************************************
;* Return if the <bit> of the <filereg> is 0.
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;* Mem   : 2
;* Clock : 2
;**************************************************************************
RETURN_IF_0	macro filereg, bit
		;==========================================================
			btfss   filereg, bit         
			return


		endm




;***** RETURN_IF_1 ********************************************************
;* Return if the <bit> of the <filereg> is 1.
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;* Mem   : 2
;* Clock : 2
;**************************************************************************
RETURN_IF_1	macro filereg, bit
		;==========================================================
			btfsc   filereg, bit         
			return


		endm




;***** BIT_SET ************************************************************
;* Set bit to 1.
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;* Mem   : 1
;* Clock : 1
;**************************************************************************
BIT_SET		macro filereg, bit
		;==========================================================
			bsf	filereg, bit         


		endm




;***** BIT_CLEAR **********************************************************
;* Set bit to 0.
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;* Mem   : 1
;* Clock : 1
;**************************************************************************
BIT_CLEAR	macro filereg, bit
		;==========================================================
			bsf	filereg, bit         


		endm




;***** BIT_TOGGLE *********************************************************
;* toggle bit
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;* Mem   : 5
;* Clock : 5
;**************************************************************************
BIT_TOGGLE	macro filereg, bit
		;==========================================================
									NOEXP
			local bt_was0, bt_was1, bt_end
									EXP
			btfss	filereg, bit
			goto	bt_was0
bt_was1			bcf	filereg, bit
			goto	bt_end
bt_was0			bsf	filereg, bit
bt_end


		endm




;***** BIT_TOGGLEW ********************************************************
;* toggle bit (use w register)
;* Param : filereg, bit        		; a bit of a filereg (a flag)
;* Mem   : 2
;* Clock : 2
;**************************************************************************
BIT_TOGGLEW	macro filereg, bit
		;==========================================================
									NOEXP
		local togglemask, loopcnt

togglemask = 1
loopcnt = 0

	while (loopcnt < bit)
loopcnt=loopcnt+1
togglemask=togglemask<<1
	endw
									EXP
			movlw	togglemask
			xorwf	filereg, f



		endm




;***** BIT_COPY ***********************************************************
;* copy a bit into another bit
;* Param : destfilereg, destbit     		; destination bit (flag)
;*	   srcfilereg,  srcbit			; source bit
;* Mem   : 3
;* Clock : 3
;**************************************************************************
BIT_COPY	macro destfilereg, destbit, srcfilereg, srcbit 
		;==========================================================
			bcf	destfilereg, destbit
			btfsc	srcfilereg, srcbit
			bsf	destfilereg, destbit


		endm




;***** BIT_INVCOPY ********************************************************
;* copy a inverted bit into another bit
;* Param : destfilereg, destbit     		; destination bit (flag)
;*	   srcfilereg,  srcbit			; source bit
;* Mem   : 3
;* Clock : 3
;**************************************************************************
BIT_INVCOPY	macro destfilereg, destbit, srcfilereg, srcbit 
		;==========================================================
			bcf	destfilereg, destbit
			btfss	srcfilereg, srcbit
			bsf	destfilereg, destbit


		endm



;***** IT_ENABLE **********************************************************
;* Enable global interrupt
;* Param : -	        		; 
;* Mem   : 1
;* Clock : 1
;**************************************************************************
IT_ENABLE	macro
		;==========================================================
			bsf	INTCON, GIE	  


		endm




;***** IT_DISABLE *********************************************************
;* Disable global interrupt
;* Param : -	        		; 
;* Mem   : 3
;* Clock : 3-7
;**************************************************************************
IT_DISABLE	macro
		;==========================================================
									NOEXP
		local	GIE_off
									EXP
GIE_off			bcf	INTCON, GIE	  
			btfsc	INTCON, GIE	; make sure bcf was not interrupted
			goto	GIE_off


		endm





;***** LOAD_ADDRESS *******************************************************
;* Load indirect address to FSR register and set IRP bit
;* Param : addr	        		; address of a file register
;* Mem   : 3
;* Clock : 3
;**************************************************************************
LOAD_ADDRESS	macro addr
		;==========================================================
			movlw	low(addr)
			movwf	FSR	  
									NOEXP
			if NUMBER_OF_BANK>2
			  if addr > 0FFh
									EXP
			ibankHI
									NOEXP
			  else
									EXP
			ibankLO
									NOEXP
			  endif
			endif
									EXP


		endm




;***** TABLE_READ *********************************************************
;* Read the n. element of the table
;* Param : table_address		; address of the table (label)
;*         indexreg			; register storing the index
;* Mem   : 7
;* Clock : 8
;**************************************************************************
TABLE_READ	macro table_address, indexreg
		;==========================================================					
			movlw	HIGH table_address
			movwf	PCLATH

			movf	indexreg,w
			addlw	LOW table_address
			skipnc
			incf	PCLATH,f
			movwf	PCL 


		endm




;***** BANK0 **************************************************************
;* Set RAM bank 
;* Param : -		       		; 
;* Mem   : 2
;* Clock : 2
;**************************************************************************
_BANK0		macro
		;==========================================================
			bank0
			bankLO


		endm



;***** BANK1 **************************************************************
;* Set RAM bank 
;* Param : -		       		; 
;* Mem   : 2
;* Clock : 2
;**************************************************************************
_BANK1		macro
		;==========================================================
			bank1
			bankLO


		endm



;***** BANK2 **************************************************************
;* Set RAM bank 
;* Param : -		       		; 
;* Mem   : 2
;* Clock : 2
;**************************************************************************
_BANK2		macro
		;==========================================================
			bank0
			bankHI


		endm



;***** BANK3 **************************************************************
;* Set RAM bank 
;* Param : -		       		; 
;* Mem   : 2
;* Clock : 2
;**************************************************************************
_BANK3		macro
		;==========================================================
			bank1
			bankHI


		endm



;**************************************************************************
	list

